@@ -32,10 +32,11 @@
 //! # Cargo.toml
 //!
 //! [dependencies]
-//! windmark = "0.1.2"
+//! windmark = "0.1.3"
+//! tokio = { version = "0.2.4", features = ["full"] }
 //!
 //! # If you would like to use the built-in logger (recommended)
-//! # windmark = { version = "0.1.2", features = ["logger"] }
+//! # windmark = { version = "0.1.3", features = ["logger"] }
 //! ```
 //!
 //! ### Implement a Windmark server
@@ -50,10 +51,10 @@
 //!   windmark::Router::new()
 //!     .set_private_key_file("windmark_private.pem")
 //!     .set_certificate_chain_file("windmark_pair.pem")
-//!     .mount("/", |_| Response::Success("Hello, World!".into()))
-//!     .set_error_handler(|_| {
+//!     .mount("/", Box::new(|_| Response::Success("Hello, World!".into())))
+//!     .set_error_handler(Box::new(|_| {
 //!       Response::PermanentFailure("This route does not exist!".into())
-//!     })
+//!     }))
 //!     .run()
 //!     .await
 //! }
@@ -69,7 +70,7 @@
 //! This project is licensed with the
 //! [GNU General Public License v3.0](https://github.com/gemrest/windmark/blob/main/LICENSE).
 
-#![feature(once_cell)]
+#![feature(once_cell, fn_traits)]
 #![deny(
   warnings,
   nonstandard_style,
@@ -89,7 +90,10 @@ pub mod utilities;
 #[macro_use]
 extern crate log;
 
-use std::{error::Error, sync::Arc};
+use std::{
+  error::Error,
+  sync::{Arc, Mutex},
+};
 
 use openssl::ssl::{self, SslAcceptor, SslMethod};
 pub use response::Response;
@@ -110,17 +114,17 @@ use crate::{
 /// response generation, panics, logging, and more.
 #[derive(Clone)]
 pub struct Router {
-  routes: matchit::Router<RouteResponse>,
-  error_handler: ErrorResponse,
+  routes: matchit::Router<Arc<Mutex<RouteResponse>>>,
+  error_handler: Arc<Mutex<ErrorResponse>>,
   private_key_file_name: String,
   certificate_chain_file_name: String,
-  header: Partial,
-  footer: Partial,
+  header: Arc<Mutex<Partial>>,
+  footer: Arc<Mutex<Partial>>,
   ssl_acceptor: Arc<SslAcceptor>,
   #[cfg(feature = "logger")]
   default_logger: bool,
-  pre_route_callback: Callback,
-  post_route_callback: Callback,
+  pre_route_callback: Arc<Mutex<Callback>>,
+  post_route_callback: Arc<Mutex<Callback>>,
   charset: String,
   language: String,
 }
@@ -179,17 +183,24 @@ impl Router {
   /// use windmark::Response;
   ///
   /// windmark::Router::new()
-  ///   .mount("/", |_| Response::Success("This is the index page!".into()))
-  ///   .mount("/test", |_| {
-  ///     Response::Success("This is a test page!".into())
-  ///   });
+  ///   .mount(
+  ///     "/",
+  ///     Box::new(|_| Response::Success("This is the index page!".into())),
+  ///   )
+  ///   .mount(
+  ///     "/test",
+  ///     Box::new(|_| Response::Success("This is a test page!".into())),
+  ///   );
   /// ```
   ///
   /// # Panics
   ///
   /// if the route cannot be mounted.
   pub fn mount(&mut self, route: &str, handler: RouteResponse) -> &mut Self {
-    self.routes.insert(route, handler).unwrap();
+    self
+      .routes
+      .insert(route, Arc::new(Mutex::new(handler)))
+      .unwrap();
 
     self
   }
@@ -199,12 +210,12 @@ impl Router {
   /// # Examples
   ///
   /// ```rust
-  /// windmark::Router::new().set_error_handler(|_| {
+  /// windmark::Router::new().set_error_handler(Box::new(|_| {
   ///   windmark::Response::Success("You have encountered an error!".into())
-  /// });
+  /// }));
   /// ```
   pub fn set_error_handler(&mut self, handler: ErrorResponse) -> &mut Self {
-    self.error_handler = handler;
+    self.error_handler = Arc::new(Mutex::new(handler));
 
     self
   }
@@ -214,12 +225,12 @@ impl Router {
   /// # Examples
   ///
   /// ```rust
-  /// windmark::Router::new().set_header(|context| {
+  /// windmark::Router::new().set_header(Box::new(|context| {
   ///   format!("This is displayed at the top of {}!", context.url.path())
-  /// });
+  /// }));
   /// ```
   pub fn set_header(&mut self, handler: Partial) -> &mut Self {
-    self.header = handler;
+    self.header = Arc::new(Mutex::new(handler));
 
     self
   }
@@ -229,12 +240,12 @@ impl Router {
   /// # Examples
   ///
   /// ```rust
-  /// windmark::Router::new().set_footer(|context| {
+  /// windmark::Router::new().set_footer(Box::new(|context| {
   ///   format!("This is displayed at the bottom of {}!", context.url.path())
-  /// });
+  /// }));
   /// ```
   pub fn set_footer(&mut self, handler: Partial) -> &mut Self {
-    self.footer = handler;
+    self.footer = Arc::new(Mutex::new(handler));
 
     self
   }
@@ -244,7 +255,7 @@ impl Router {
   /// # Examples
   ///
   /// ```rust
-  /// windmark::Router::new().run(); 
+  /// windmark::Router::new().run();
   /// ```
   ///
   /// # Panics
@@ -272,7 +283,7 @@ impl Router {
       match stream {
         Ok(stream) => {
           let acceptor = acceptor.clone();
-          let self_clone = self.clone();
+          let mut self_clone = self.clone();
 
           tokio::spawn(async move {
             match tokio_openssl::accept(&acceptor, stream).await {
@@ -292,8 +303,9 @@ impl Router {
     Ok(())
   }
 
+  #[allow(clippy::too_many_lines)]
   async fn handle(
-    &self,
+    &mut self,
     stream: &mut tokio_openssl::SslStream<tokio::net::TcpStream>,
   ) -> Result<(), Box<dyn std::error::Error>> {
     let mut buffer = [0u8; 1024];
@@ -315,20 +327,22 @@ impl Router {
 
     let route = self.routes.at(url.path());
 
-    (self.pre_route_callback)(stream.get_ref(), &url, {
-      if let Ok(route) = &route {
-        Some(&route.params)
-      } else {
-        None
-      }
-    });
+    (*self.pre_route_callback).lock().unwrap().call_mut((
+      stream.get_ref(),
+      &url,
+      {
+        if let Ok(route) = &route {
+          Some(&route.params)
+        } else {
+          None
+        }
+      },
+    ));
 
     if let Ok(ref route) = route {
       header = {
-        let header = (self.header)(RouteContext::new(
-          stream.get_ref(),
-          &url,
-          &route.params,
+        let header = (*self.header).lock().unwrap().call_mut((
+          RouteContext::new(stream.get_ref(), &url, &route.params),
         ));
 
         if header.is_empty() {
@@ -338,10 +352,8 @@ impl Router {
         }
       };
       footer = {
-        let footer = (self.footer)(RouteContext::new(
-          stream.get_ref(),
-          &url,
-          &route.params,
+        let footer = (*self.footer).lock().unwrap().call_mut((
+          RouteContext::new(stream.get_ref(), &url, &route.params),
         ));
 
         if footer.is_empty() {
@@ -352,17 +364,20 @@ impl Router {
       };
       content = {
         to_value_set_status(
-          (route.value)(RouteContext::new(
+          (*route.value).lock().unwrap().call_mut((RouteContext::new(
             stream.get_ref(),
             &url,
             &route.params,
-          )),
+          ),)),
           &mut response_status,
         )
       };
     } else {
       content = to_value_set_status(
-        (self.error_handler)(ErrorContext::new(stream.get_ref(), &url)),
+        (*self.error_handler)
+          .lock()
+          .unwrap()
+          .call_mut((ErrorContext::new(stream.get_ref(), &url),)),
         &mut response_status,
       );
     }
@@ -395,13 +410,17 @@ impl Router {
       )
       .await?;
 
-    (self.post_route_callback)(stream.get_ref(), &url, {
-      if let Ok(route) = &route {
-        Some(&route.params)
-      } else {
-        None
-      }
-    });
+    (*self.post_route_callback).lock().unwrap().call_mut((
+      stream.get_ref(),
+      &url,
+      {
+        if let Ok(route) = &route {
+          Some(&route.params)
+        } else {
+          None
+        }
+      },
+    ));
 
     stream.shutdown().await?;
 
@@ -506,15 +525,17 @@ impl Router {
   /// ```rust
   /// use log::info;
   ///
-  /// windmark::Router::new().set_pre_route_callback(|stream, _url, _| {
-  ///   info!(
-  ///     "accepted connection from {}",
-  ///     stream.peer_addr().unwrap().ip(),
-  ///   )
-  /// });
+  /// windmark::Router::new().set_pre_route_callback(Box::new(
+  ///   |stream, _url, _| {
+  ///     info!(
+  ///       "accepted connection from {}",
+  ///       stream.peer_addr().unwrap().ip(),
+  ///     )
+  ///   },
+  /// ));
   /// ```
   pub fn set_pre_route_callback(&mut self, callback: Callback) -> &mut Self {
-    self.pre_route_callback = callback;
+    self.pre_route_callback = Arc::new(Mutex::new(callback));
 
     self
   }
@@ -526,15 +547,17 @@ impl Router {
   /// ```rust
   /// use log::info;
   ///
-  /// windmark::Router::new().set_post_route_callback(|stream, _url, _| {
-  ///   info!(
-  ///     "closed connection from {}",
-  ///     stream.peer_addr().unwrap().ip(),
-  ///   )
-  /// });
+  /// windmark::Router::new().set_post_route_callback(Box::new(
+  ///   |stream, _url, _| {
+  ///     info!(
+  ///       "closed connection from {}",
+  ///       stream.peer_addr().unwrap().ip(),
+  ///     )
+  ///   },
+  /// ));
   /// ```
   pub fn set_post_route_callback(&mut self, callback: Callback) -> &mut Self {
-    self.post_route_callback = callback;
+    self.post_route_callback = Arc::new(Mutex::new(callback));
 
     self
   }
@@ -550,12 +573,15 @@ impl Router {
   /// use windmark::Response;
   ///
   /// windmark::Router::new().attach(|r| {
-  ///   r.mount("/module", |_| Response::Success("This is a module!".into()));
-  ///   r.set_error_handler(|_| {
+  ///   r.mount(
+  ///     "/module",
+  ///     Box::new(|_| Response::Success("This is a module!".into())),
+  ///   );
+  ///   r.set_error_handler(Box::new(|_| {
   ///     Response::NotFound(
   ///       "This error handler has been implemented by a module!".into(),
   ///     )
-  ///   });
+  ///   }));
   /// });
   /// ```
   pub fn attach<F>(&mut self, mut module: F) -> &mut Self
@@ -599,15 +625,15 @@ impl Default for Router {
   fn default() -> Self {
     Self {
       routes: matchit::Router::new(),
-      error_handler: |_| {
+      error_handler: Arc::new(Mutex::new(Box::new(|_| {
         Response::NotFound(
           "This capsule has not implemented an error handler...".to_string(),
         )
-      },
+      }))),
       private_key_file_name: "".to_string(),
       certificate_chain_file_name: "".to_string(),
-      header: |_| "".to_string(),
-      footer: |_| "".to_string(),
+      header: Arc::new(Mutex::new(Box::new(|_| "".to_string()))),
+      footer: Arc::new(Mutex::new(Box::new(|_| "".to_string()))),
       ssl_acceptor: Arc::new(
         SslAcceptor::mozilla_intermediate(SslMethod::tls())
           .unwrap()
@@ -615,8 +641,8 @@ impl Default for Router {
       ),
       #[cfg(feature = "logger")]
       default_logger: false,
-      pre_route_callback: |_, _, _| {},
-      post_route_callback: |_, _, _| {},
+      pre_route_callback: Arc::new(Mutex::new(Box::new(|_, _, _| {}))),
+      post_route_callback: Arc::new(Mutex::new(Box::new(|_, _, _| {}))),
       charset: "utf-8".to_string(),
       language: "en".to_string(),
     }
